; ********************************************************************************
; zcode PWM calculation and update functions
;
;-------------------------------------------
; Fade Direction
; 
_colActivate  	btfss  	flags, rxNewCol	; is colour data stale?
        	return  		; exit if it is
      	bcf     	flags, rxNewCol	; else clear flag
	bsf	T1CON, TMR1ON	

	movfw   	fadeLatch	; get new fade rate
	bz	_noFade	; do noFade code if fade rate == 0

	movwf   	fadeRate	; else save fade rate
	movwf	fadeCounter	; 
	movfw   	servo1Latch 	; transfer colour data from
        	movwf   	servo1NEW 	; receive latch to new data
        	movfw   	servo2Latch 
        	movwf   	servo2NEW 
        	movfw   	servo3Latch 
        	movwf   	servo3NEW	
			; and then compute fade rate

;-------------------------------------------
; Calculate the difference between current and new
; PWM value for each colour. Set fade direction to +/-1
	
	
_calcFade	movlw	(1<<s1done | 1<<s2done | 1<<s3done)
	movwf	fadeflag	; preset fade flag
	movlw	0x01	; preset fade direction to +1 for each colour
	movwf	s1dir
	movwf	s2dir
	movwf	s3dir
	
	movfw	s1pos
	subwf	servo1NEW,W	; subtract current from new (W = current - new)
	bc	_s1abs	; branch if C=1  (current > new)
	decf	s1dir,F	; rdir is 1, decrement twice
	decf	s1dir,F	; to make rdir -1.
	sublw	0x00	; sub W from 0 to make W positive
_s1abs	movwf	s1dif	; save difference between current and new

	movfw	s2pos
	subwf	servo2NEW,W	; subtract current from new (W = current - new)
	bc	_s2abs	; branch if C=1  (current > new)
	decf	s2dir,F	; rdir is 1, decrement twice
	decf	s2dir,F	; to make rdir -1.
	sublw	0x00	; sub W from 0 to make W positive
_s2abs	movwf	s2dif	; save difference between current and new

	movfw	s3pos
	subwf	servo3NEW,W	; subtract current from new (W = current - new)
	bc	_s3abs	; branch if C=1  (current > new)
	decf	s3dir,F	; rdir is 1, decrement twice
	decf	s3dir,F	; to make rdir -1.
	sublw	0x00	; sub W from 0 to make W positive
_s3abs	movwf	s3dif	; save difference between current and new


;-------------------------------------------
; Find largest difference between current and new RGB values
; 	
	
_calcLargest	movfw	s1dif
	subwf	s2dif,W
	bc	_chkGB
	
	movfw	s1dif
	subwf	s3dif,W
	movfw	s3dif
	bc	_done
	movfw	s1dif
	goto	_done
	
_chkGB	movfw	s2dif
	subwf	s3dif,W
	movfw	s3dif
	skpc	
	movfw	s2dif

_done	sublw	0x00
	movwf	CntBase
	movwf	s1cnt
	movwf	s2cnt
	movwf	s3cnt
	
	return

;-----------------------------------------------------
; Test if current fade completed
	
_fadeState	movf	fadeflag,F	; test if fadeflag is zero
	skpnz
	return
	
;------------------------------------------------------
; fade	
	
_fading	decfsz	fadeCounter,F	; decrement fade counter
	return		; return if not zero
	movfw	fadeRate	; else reload fade counter
	movwf	fadeCounter

;----------------------------------------
; Fade stepper
;

_s1slew	movfw	servo1NEW
	xorwf	s1pos,W
	bnz	$+3
	bcf	fadeflag,s1done
	goto	_s2slew

	movfw	s1dif
	addwf	s1cnt,F
	skpc	
	goto	_s2slew
	movfw	CntBase
	addwf	s1cnt,F
	movfw	s1dir
	addwf	s1pos,F
	
_s2slew	movfw	servo2NEW
	xorwf	s2pos,W
	bnz	$+3
	bcf	fadeflag,s2done
	goto	_s3slew

	movfw	s2dif
	addwf	s2cnt,F
	skpc	
	goto	_s3slew
	movfw	CntBase
	addwf	s2cnt,F
	movfw	s2dir
	addwf	s2pos,F

_s3slew 	movfw	servo3NEW
	xorwf	s3pos,W
	bnz	$+3
	bcf	fadeflag,s3done
	return
	
	movfw	s3dif
	addwf	s3cnt,F
	skpc	
	return	
	movfw	CntBase
	addwf	s3cnt,F
	movfw	s3dir
	addwf	s3pos,F
	return

;----------------------------------------
; No Fade colour transfer
;
; if faderate == 0 transfer colour data directly without any fading.

_noFade	movfw	servo1Latch	; No fade so load
	movwf	s1pos	; pwm variables
	movfw	servo2Latch	; with new data
	movwf	s2pos	; to get instant
	movfw	servo3Latch	; colour change
	movwf	s3pos	; ...
	clrf	fadeflag	; clear fade flags so code doesn't attempt to fade
	return		; return