;-------------------------------------------------------------------------------------------------------------------------------
; Serial data receive function
; receives, validates, buffers with error checking and handling
; complete valid data frames are transferred to secondary buffer and rxDataReady flag bit set


_serialRX	movfw	RCSTA	; test for frame and overrun errors
	andlw	(1<<FERR | 1<<OERR)
	bnz	_rxError
	btfss	PIR1, RCIF	; is data ready in RX buffer
	return

	movfw	RCREG	; read received data
	movwf	INDF	; save in rxBuffer
	addwf	Chksum,F	; update checksum
	
	incf	rxPSD,W	; test if Start Frame Delimiter valid (0xFF)
	bz	_goodPSD	; goto error handler if SFD invalid
	addlw	0x01	; test if Start Frame Delimiter valid (OxFE)
	bnz	_rxError	; goto error handler if SFD invalid

	
_goodPSD	incf	FSR,F	; increment rx buffer pointer, skip next if 7 bytes received
	movfw	FSR
	xorlw	rxBufferBase + 7
	skpz
	goto	_serialRX	; test for more data in receive register 

	movf	Chksum,F	; test Chksum == 0
	bnz	_chkSumError	; goto error handler if checksum invalid

	movfw	myAddress	; test for my address
	xorwf	rxAddress,W
	bz	_forMe
	incf	rxAddress,W	; test for broadcast address
	bz	_forMe
	
	movfw	myGroupRow	; test for group Row address
	xorwf	rxAddress,W
	bz	_forMe	; packet is for this device

	movfw	myGroupCol	; test for group Column address
	xorwf	rxAddress,W
	bnz	_initBuffer	; packet not for this device, don't process further	

	;move data from receive buffer to working data buffer
_forMe	btfss	flags, pirate
	clrf	sleepTimerHi
	btfss	flags2, pwmShutdown
	bsf	INTCON, GIE

	btfsc	rxPSD,0	; test bit 0 of rxPSD
	goto	_moveColour	; if bit 0 == 1 rx colour data
_moveCmd	movfw	rxData0	
	movwf	Data0
	movfw	rxData1	
	movwf	Data1
	movfw	rxData2
	movwf	Data2
	movfw	rxData3
	movwf	Data3
		
	bsf	flags, rxNewCmd	; Set data ready flag
	clrf	Chksum	; initialise checksum
	movlw	rxBufferBase	; initialise receive buffer index pointer
	movwf	FSR
	goto	_serialRX	; test for more data in receive register 

_moveColour	movfw	rxData0
	movwf	servo1Latch
	movfw	rxData1
	movwf	servo2Latch
	movfw	rxData2
	movwf	servo3Latch
	movfw	rxData3
	movwf	fadeLatch
	bsf	flags, rxNewCol	; set Servo Data ready flag

_initBuffer	clrf	Chksum	; initialise checksum
	movlw	rxBufferBase	; initialise receive buffer index pointer
	movwf	FSR
	goto	_serialRX	; test for more data in receive register 


_rxError	;goto	_debug
	btfsc	RCSTA,FERR	; test for framing error
	movfw	RCREG	; and clear if present
	
	btfsc	RCSTA,OERR	; test for overrun error
	bcf	RCSTA,CREN	; and clear if present
	bsf	RCSTA,CREN


;
_chkSumError	clrf	Chksum	; clear checksum variable
	movlw	rxBufferBase	; and reset rx buffer
	movwf	FSR	; to start of packet
	goto	_serialRX	


