;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; zservo690a_reset.inc
; Servo driver reset and initialisation 
; Initialisation for 16F690
; 09/10/2007
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_main	clrf	STATUS
	

	bsf	STATUS,RP1
	clrf	ANSEL	; Set I/O ports for digital mode
	clrf	ANSELH
	bcf	STATUS,RP1
	bank1
	movlw	b'01110000'	; set OSCCON for internal RC oscillator
	movwf	OSCCON	; running at 8Mhz
	
	movlw	b'00000000'	; enable weak pull up on PORTA+B
	movwf	OPTION_REG

	movlw	b'11111111'
	movwf	TRISA
	movlw	b'11110111'
	movwf	WPUA

	
	
	movlw	b'00000100'	; enable High Speed Bit Rate
	movwf	TXSTA	; and leave serial TX disabled
	
	bsf	PIE1, TMR1IE	; enable Timer 1 interrupts

	movlw	.125
	movwf	PR2

	movlw	b'01110000'
	movwf	TRISB
	bsf	STATUS, RP1	
	bcf	STATUS, RP0	; set to bank 2
	movwf	WPUB
	bcf	STATUS, RP1
	bcf	STATUS, RP0 	; set to bank 0
	; bank 0 for here
	
	movlw	b'10000000'	; Enable serial port
	movwf	RCSTA
	
	movlw	0x30	; set T1 PS 8:1
	movwf	T1CON
	clrf	TMR1L
	movlw	0xF0
	movwf	TMR1H
	bcf	PIR1, TMR1IF

	btfss	sw2	; test if sw2 held down at reset
	goto	_EEPROMbadsum	; initialise EEPROM if input sw2 down


	call	_EEPROMboot	; read settings from EEPROM

	movfw	eepromAddress	; load saved device address
	movwf	myAddress	; initialise device address variable
	andlw	0x0F	; calculate device Group Column Address
	iorlw	0x90
	movwf	myGroupCol	; and initialise group address variable
	
	swapf	myAddress,W	; calculate device Group Row Address
	andlw	0x0F
	iorlw	0x80
	movwf	myGroupRow

	movfw	eepromInactive
	movwf	activeTmrEn	; initialise inactivity timeout flag
	
	bank1
	movfw	eepromBitRate	; read saved bit rate
	movwf	savedBitRate	; keep copy of saved bit rate
	call	_bitRateLookup
	movwf	SPBRG	; and initialise SPBRG register
	xorlw	.103	; test if SPBRG value is .103 (1.2Kbps)
	skpnz		;
	bcf	TXSTA, BRGH	; Requires BRGH=0 for low speed
	clrf	BAUDCTL
	bank0
		
	movlw	b'01111101'
	movwf	T2CON
	
	clrf	fadeflag
	clrf	flags	; initialise flag variable
	clrf	flags2	; initialise flags2 variable
	clrf	Chksum	; initialise checksum
	call	_antiPiracy	; in zcode_cmdfuncs.inc
	addlw	-'z'	; test if 'z' returned
	skpz		; skip next if it was
	bsf	flags, pirate	; else set the pirate flag

	btfsc	segDriveLevel
	bsf	flags2, LEDtype	; set 7 seg drive for CA or CC [0 = CC, 1 =CA ]	

	movlw	b'00001000'
	movwf	servoState

	movlw	rxBufferBase	; initialise rx buffer index pointer
	movwf	FSR
	clrf	PCLATH	; set PCLATH to page 0
	
	clrf	PORTC
	bank1
	movlw	b'00000000'
	movwf	TRISC
	bank0
	bsf	outEN	
	bcf	rClk
	

	bsf	INTCON, PEIE	; enable Peripheral interrupt
	bsf	INTCON, GIE	; enable Global interrupt
		
	

	clrf	column	; 7seg digit drive pointer
	bsf	RCSTA, CREN	; enable serial port for continuos receive

	;goto	_frontPanelTest
	clrf	fpState
	clrf	sleepTimerHi	
	clrf	sleepTimerLo

;---------------------------------------------------------------------------
; zcode_core
;
; core code loop
;
_coreLoop	btfsc	PIR1, RCIF	; is data ready in RX buffer
	call	_serialRX
	
	btfsc	PIR1,TMR2IF	; 4mS timer tick
	call	_timerTick
	
	btfsc	PIR1, RCIF	; is data ready in RX buffer
	call	_serialRX

	btfsc	flags, rxNewCmd	; has new command been received
	call	_command

	goto	_coreLoop
	
_timerTick	bcf	PIR1,TMR2IF	; clear timer 1 int' flag
	
	; Test for sw1 held for over 1 second 
	btfsc	sw1	; Skip next if sw1 down
	clrf	sw1timer	; clear sw1Timer if sw1 up
	btfss	sw1	; Skip next if sw1 up
	incfsz	sw1timer,F	; Increment sw1Timer if sw1 up, skip next if rolled-over
	goto	_fader	; goto _fader


_fpControl 	call	_dispAddr	; intialise 7seg display with current address
	clrf	fpState	; clear fpState (address =0,  bit rate =1, save mode =2)
	clrf	sw1timer	
	clrf	sw2timer
	bsf	flags2, sw1Last	; preset sw1Last flag
	
	btfss	PIR1, TMR2IF	; wait for sw1 to be released
	goto	$-1	
	bcf	PIR1, TMR2IF
	call	_7segDriver	; and multiplex 7 seg display
	btfss	sw1
	goto	$-5	
	

_fpLoop	btfss	PIR1, TMR2IF	
	goto	$-1
	bcf	PIR1, TMR2IF
	call	_7segDriver	; multiplex 7 seg display
	call	_testSw2	; test switch 2
	call	_testSw1	; test switch 1
	btfss	flags2, exitFP	; do we need to exit front panel setup?
	goto	_fpLoop	; no, then loop 

	bcf	flags2, exitFP	; yes, then clear flag
	; Hi Z 7seg segment drive
	bsf	outEN

	; wait for sw1 to be released 
	btfss	PIR1, TMR2IF	
	goto	$-1
	bcf	PIR1, TMR2IF	
	btfss	sw1
	goto	$-5

		
	; fade / hold state function select
	
_fader	movf	fadeflag,F	; test fade flag
	skpz		; skip next if zero
	call	_fading	; else call RGB fade function

	btfsc	flags, pirate
	goto	_sleepTimer
	movf	activeTmrEn,F
	skpnz
	return

_sleepTimer	incf	sleepTimerLo,F	; increment 16 bit sleep timer
	skpnz	
	incfsz	sleepTimerHi,F	
	return
	
	bcf	INTCON, GIE	; if sleep timer rolled over
	bcf	INTCON, GIE	; stop PWM interrupt
	bcf	T1CON, TMR1ON
	movfw	ServoPort
	andlw	~(1<<servo1 | 1<<servo2 | 1<<servo3)
	movwf	ServoPort
	bsf	PIR1, TMR1IF
	return


	



